# Generated by Django 5.2.11 on 2026-02-08 23:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('legacy_data', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='legacycalculationid',
            options={'managed': False, 'verbose_name': 'Generic Calculation ID', 'verbose_name_plural': 'Generic Calculation IDs'},
        ),
        migrations.AlterModelOptions(
            name='legacycalculationlog',
            options={'managed': False, 'ordering': ['-timestamp'], 'verbose_name': 'Generic Calculation Log', 'verbose_name_plural': 'Generic Calculation Logs'},
        ),
        migrations.AlterModelOptions(
            name='legacyuserchangelog',
            options={'managed': False, 'ordering': ['-timestamp'], 'verbose_name': 'Generic User Change Log', 'verbose_name_plural': 'Generic User Change Logs'},
        ),
    ]
